@@ -7,7 +7,7 @@ import sys
 from gettext import gettext as _
 from io import TextIOWrapper
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Sequence
+from typing import Any, Dict, List, Optional, Sequence, Union
 from warnings import warn
 
 from . import __version__, api, files, sections
@@ -15,12 +15,9 @@ from .exceptions import FileSkipped, ISortError, UnsupportedEncoding
 from .format import create_terminal_printer
 from .logo import ASCII_ART
 from .profiles import profiles
-from .settings import VALID_PY_TARGETS, Config, WrapModes
-
-try:
-    from .setuptools_commands import ISortCommand  # noqa: F401
-except ImportError:
-    pass
+from .settings import VALID_PY_TARGETS, Config, find_all_configs
+from .utils import Trie
+from .wrap_modes import WrapModes
 
 DEPRECATED_SINGLE_DASH_ARGS = {
     "-ac",
@@ -123,11 +120,13 @@ def _print_hard_fail(
 ) -> None:
     """Fail on unrecoverable exception with custom message."""
     message = message or (
-        f"Unrecoverable exception thrown when parsing {offending_file or ''}!"
+        f"Unrecoverable exception thrown when parsing {offending_file or ''}! "
         "This should NEVER happen.\n"
         "If encountered, please open an issue: https://github.com/PyCQA/isort/issues/new"
     )
-    printer = create_terminal_printer(color=config.color_output)
+    printer = create_terminal_printer(
+        color=config.color_output, error=config.format_error, success=config.format_success
+    )
     printer.error(message)
 
 
@@ -139,8 +138,8 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "Use `-` as the first argument to represent stdin. Use --interactive to use the pre 5.0.0 "
         "interactive behavior."
         " "
-        "If you've used isort 4 but are new to isort 5, see the upgrading guide:"
-        "https://pycqa.github.io/isort/docs/upgrade_guides/5.0.0/.",
+        "If you've used isort 4 but are new to isort 5, see the upgrading guide: "
+        "https://pycqa.github.io/isort/docs/upgrade_guides/5.0.0.html",
         add_help=False,  # prevent help option from appearing in "optional arguments" group
     )
 
@@ -209,8 +208,8 @@ def _build_arg_parser() -> argparse.ArgumentParser:
     )
     general_group.add_argument(
         "--overwrite-in-place",
-        help="Tells isort to overwrite in place using the same file handle."
-        "Comes at a performance and memory usage penalty over it's standard "
+        help="Tells isort to overwrite in place using the same file handle. "
+        "Comes at a performance and memory usage penalty over its standard "
         "approach but ensures all file flags and modes stay unchanged.",
         dest="overwrite_in_place",
         action="store_true",
@@ -225,7 +224,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--show-files",
         dest="show_files",
         action="store_true",
-        help="See the files isort will be ran against with the current config options.",
+        help="See the files isort will be run against with the current config options.",
     )
     general_group.add_argument(
         "--df",
@@ -261,6 +260,22 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         help="Explicitly set the settings path or file instead of auto determining "
         "based on file location.",
     )
+    general_group.add_argument(
+        "--cr",
+        "--config-root",
+        dest="config_root",
+        help="Explicitly set the config root for resolving all configs. When used "
+        "with the --resolve-all-configs flag, isort will look at all sub-folders "
+        "in this config root to resolve config files and sort files based on the "
+        "closest available config(if any)",
+    )
+    general_group.add_argument(
+        "--resolve-all-configs",
+        dest="resolve_all_configs",
+        action="store_true",
+        help="Tells isort to resolve the configs for all sub-directories "
+        "and sort files in terms of its closest config files.",
+    )
     general_group.add_argument(
         "--profile",
         dest="profile",
@@ -278,7 +293,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
     general_group.add_argument(
         "-j",
         "--jobs",
-        help="Number of files to process in parallel.",
+        help="Number of files to process in parallel. Negative value means use number of CPUs.",
         dest="jobs",
         type=int,
         nargs="?",
@@ -297,6 +312,23 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="Tells isort to apply changes interactively.",
     )
+    general_group.add_argument(
+        "--format-error",
+        dest="format_error",
+        help="Override the format used to print errors.",
+    )
+    general_group.add_argument(
+        "--format-success",
+        dest="format_success",
+        help="Override the format used to print success.",
+    )
+    general_group.add_argument(
+        "--srx",
+        "--sort-reexports",
+        dest="sort_reexports",
+        action="store_true",
+        help="Automatically sort all re-exports (module level __all__ collections)",
+    )
 
     target_group.add_argument(
         "files", nargs="*", help="One or more Python source files that need their imports sorted."
@@ -338,7 +370,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
     target_group.add_argument(
         "--extend-skip-glob",
         help="Additional files that isort should skip over (extending --skip-glob).",
-        dest="skip_glob",
+        dest="extend_skip_glob",
         action="append",
     )
     target_group.add_argument(
@@ -346,7 +378,8 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--skip-gitignore",
         action="store_true",
         dest="skip_gitignore",
-        help="Treat project as a git repository and ignore files listed in .gitignore",
+        help="Treat project as a git repository and ignore files listed in .gitignore."
+        "\nNOTE: This requires git to be installed and accessible from the same shell as isort.",
     )
     target_group.add_argument(
         "--ext",
@@ -354,13 +387,13 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--supported-extension",
         dest="supported_extensions",
         action="append",
-        help="Specifies what extensions isort can be ran against.",
+        help="Specifies what extensions isort can be run against.",
     )
     target_group.add_argument(
         "--blocked-extension",
         dest="blocked_extensions",
         action="append",
-        help="Specifies what extensions isort can never be ran against.",
+        help="Specifies what extensions isort can never be run against.",
     )
     target_group.add_argument(
         "--dont-follow-links",
@@ -377,7 +410,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--allow-root",
         action="store_true",
         default=False,
-        help="Tells isort not to treat / specially, allowing it to be ran against the root dir.",
+        help="Tells isort not to treat / specially, allowing it to be run against the root dir.",
     )
 
     output_group.add_argument(
@@ -453,6 +486,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--ff",
         "--from-first",
         dest="from_first",
+        action="store_true",
         help="Switches the typical ordering preference, "
         "showing from imports first then straight ones.",
     )
@@ -463,7 +497,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         const=2,
         type=int,
         dest="force_grid_wrap",
-        help="Force number of from imports (defaults to 2 when passed as CLI flag without value)"
+        help="Force number of from imports (defaults to 2 when passed as CLI flag without value) "
         "to be grid wrapped regardless of line "
         "length. If 0 is passed in (the global default) only line length is considered.",
     )
@@ -474,6 +508,9 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         dest="indent",
         type=str,
     )
+    output_group.add_argument(
+        "--lbi", "--lines-before-imports", dest="lines_before_imports", type=int
+    )
     output_group.add_argument(
         "--lai", "--lines-after-imports", dest="lines_after_imports", type=int
     )
@@ -568,6 +605,12 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="Reverses the ordering of imports.",
     )
+    output_group.add_argument(
+        "--sort-order",
+        dest="sort_order",
+        help="Specify sorting function. Can be built in (natural[default] = force numbers "
+        "to be sequential, native = Python's built-in sorted function) or an installable plugin.",
+    )
     inline_args_group.add_argument(
         "--sl",
         "--force-single-line-imports",
@@ -672,6 +715,12 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         dest="star_first",
         action="store_true",
     )
+    output_group.add_argument(
+        "--split-on-trailing-comma",
+        help="Split imports list followed by a trailing comma into VERTICAL_HANGING_INDENT mode",
+        dest="split_on_trailing_comma",
+        action="store_true",
+    )
 
     section_group.add_argument(
         "--sd",
@@ -684,8 +733,9 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         "--os",
         dest="only_sections",
         action="store_true",
-        help="Causes imports to be sorted only based on their sections like STDLIB,THIRDPARTY etc. "
-        "Imports are unaltered and keep their relative positions within the different sections.",
+        help="Causes imports to be sorted based on their sections like STDLIB, THIRDPARTY, etc. "
+        "Within sections, the imports are ordered by their import style and the imports with "
+        "the same style maintain their relative positions.",
     )
     section_group.add_argument(
         "--ds",
@@ -761,7 +811,8 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         dest="src_paths",
         action="append",
         help="Add an explicitly defined source path "
-        "(modules within src paths have their imports automatically categorized as first_party).",
+        "(modules within src paths have their imports automatically categorized as first_party)."
+        " Glob expansion (`*` and `**`) is supported for this option.",
     )
     section_group.add_argument(
         "-b",
@@ -784,7 +835,7 @@ def _build_arg_parser() -> argparse.ArgumentParser:
         help="Force isort to recognize a module as part of Python's internal future compatibility "
         "libraries. WARNING: this overrides the behavior of __future__ handling and therefore"
         " can result in code that can't execute. If you're looking to add dependencies such "
-        "as six a better option is to create a another section below --future using custom "
+        "as six, a better option is to create another section below --future using custom "
         "sections. See: https://github.com/PyCQA/isort#custom-sections-and-ordering and the "
         "discussion here: https://github.com/PyCQA/isort/issues/1463.",
     )
@@ -906,17 +957,17 @@ def parse_args(argv: Optional[Sequence[str]] = None) -> Dict[str, Any]:
     return arguments
 
 
-def _preconvert(item):
+def _preconvert(item: Any) -> Union[str, List[Any]]:
     """Preconverts objects from native types into JSONifyiable types"""
     if isinstance(item, (set, frozenset)):
         return list(item)
     if isinstance(item, WrapModes):
-        return item.name
+        return str(item.name)
     if isinstance(item, Path):
         return str(item)
     if callable(item) and hasattr(item, "__name__"):
-        return item.__name__
-    raise TypeError("Unserializable object {} of type {}".format(item, type(item)))
+        return str(item.__name__)
+    raise TypeError(f"Unserializable object {item} of type {type(item)}")
 
 
 def identify_imports_main(
@@ -1053,10 +1104,15 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
     stream_filename = config_dict.pop("filename", None)
     ext_format = config_dict.pop("ext_format", None)
     allow_root = config_dict.pop("allow_root", None)
+    resolve_all_configs = config_dict.pop("resolve_all_configs", False)
     wrong_sorted_files = False
     all_attempt_broken = False
     no_valid_encodings = False
 
+    config_trie: Optional[Trie] = None
+    if resolve_all_configs:
+        config_trie = find_all_configs(config_dict.pop("config_root", "."))
+
     if "src_paths" in config_dict:
         config_dict["src_paths"] = {
             Path(src_path).resolve() for src_path in config_dict.get("src_paths", ())
@@ -1071,9 +1127,10 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
         if show_files:
             sys.exit("Error: can't show files for streaming input.")
 
+        input_stream = sys.stdin if stdin is None else stdin
         if check:
             incorrectly_sorted = not api.check_stream(
-                input_stream=sys.stdin if stdin is None else stdin,
+                input_stream=input_stream,
                 config=config,
                 show_diff=show_diff,
                 file_path=file_path,
@@ -1082,22 +1139,30 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
 
             wrong_sorted_files = incorrectly_sorted
         else:
-            api.sort_stream(
-                input_stream=sys.stdin if stdin is None else stdin,
-                output_stream=sys.stdout,
-                config=config,
-                show_diff=show_diff,
-                file_path=file_path,
-                extension=ext_format,
-            )
+            try:
+                api.sort_stream(
+                    input_stream=input_stream,
+                    output_stream=sys.stdout,
+                    config=config,
+                    show_diff=show_diff,
+                    file_path=file_path,
+                    extension=ext_format,
+                    raise_on_skip=False,
+                )
+            except FileSkipped:
+                sys.stdout.write(input_stream.read())
     elif "/" in file_names and not allow_root:
-        printer = create_terminal_printer(color=config.color_output)
+        printer = create_terminal_printer(
+            color=config.color_output, error=config.format_error, success=config.format_success
+        )
         printer.error("it is dangerous to operate recursively on '/'")
         printer.error("use --allow-root to override this failsafe")
         sys.exit(1)
     else:
         if stream_filename:
-            printer = create_terminal_printer(color=config.color_output)
+            printer = create_terminal_printer(
+                color=config.color_output, error=config.format_error, success=config.format_success
+            )
             printer.error("Filename override is intended only for stream (-) sorting.")
             sys.exit(1)
         skipped: List[str] = []
@@ -1135,6 +1200,7 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
                     ask_to_apply=ask_to_apply,
                     write_to_stdout=write_to_stdout,
                     extension=ext_format,
+                    config_trie=config_trie,
                 ),
                 file_names,
             )
@@ -1149,6 +1215,7 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
                     show_diff=show_diff,
                     write_to_stdout=write_to_stdout,
                     extension=ext_format,
+                    config_trie=config_trie,
                 )
                 for file_name in file_names
             )
@@ -1178,9 +1245,10 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
         if num_skipped and not config.quiet:
             if config.verbose:
                 for was_skipped in skipped:
-                    warn(
-                        f"{was_skipped} was skipped as it's listed in 'skip' setting"
-                        " or matches a glob in 'skip_glob' setting"
+                    print(
+                        f"{was_skipped} was skipped as it's listed in 'skip' setting, "
+                        "matches a glob in 'skip_glob' setting, or is in a .gitignore file with "
+                        "--skip-gitignore enabled."
                     )
             print(f"Skipped {num_skipped} files")
 
@@ -1209,7 +1277,7 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
             )
         warn(
             "W0500: Please see the 5.0.0 Upgrade guide: "
-            "https://pycqa.github.io/isort/docs/upgrade_guides/5.0.0/"
+            "https://pycqa.github.io/isort/docs/upgrade_guides/5.0.0.html"
         )
 
     if wrong_sorted_files:
@@ -1219,7 +1287,9 @@ def main(argv: Optional[Sequence[str]] = None, stdin: Optional[TextIOWrapper] =
         sys.exit(1)
 
     if no_valid_encodings:
-        printer = create_terminal_printer(color=config.color_output)
+        printer = create_terminal_printer(
+            color=config.color_output, error=config.format_error, success=config.format_success
+        )
         printer.error("No valid encodings.")
         sys.exit(1)
 
